/****************************************************************************
** Meta object code from reading C++ file 'executejob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kauth-6.26.0/src/executejob.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'executejob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5KAuth10ExecuteJobE_t {};
} // unnamed namespace

template <> constexpr inline auto KAuth::ExecuteJob::qt_create_metaobjectdata<qt_meta_tag_ZN5KAuth10ExecuteJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KAuth::ExecuteJob",
        "newData",
        "",
        "QVariantMap",
        "data",
        "statusChanged",
        "KAuth::Action::AuthStatus",
        "status",
        "doExecuteAction",
        "doAuthorizeAction",
        "actionPerformedSlot",
        "action",
        "KAuth::ActionReply",
        "reply",
        "progressStepSlot",
        "i",
        "statusChangedSlot",
        "kill",
        "KillVerbosity",
        "verbosity"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newData'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(KAuth::Action::AuthStatus)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'doExecuteAction'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'doAuthorizeAction'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'actionPerformedSlot'
        QtMocHelpers::SlotData<void(const QString &, const KAuth::ActionReply &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'progressStepSlot'
        QtMocHelpers::SlotData<void(const QString &, int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { QMetaType::Int, 15 },
        }}),
        // Slot 'statusChangedSlot'
        QtMocHelpers::SlotData<void(const QString &, KAuth::Action::AuthStatus)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'kill'
        QtMocHelpers::SlotData<bool(KillVerbosity)>(17, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'kill'
        QtMocHelpers::SlotData<bool()>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExecuteJob, qt_meta_tag_ZN5KAuth10ExecuteJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KAuth::ExecuteJob::staticMetaObject = { {
    QMetaObject::SuperData::link<KJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KAuth10ExecuteJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KAuth10ExecuteJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5KAuth10ExecuteJobE_t>.metaTypes,
    nullptr
} };

void KAuth::ExecuteJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExecuteJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newData((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 1: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<KAuth::Action::AuthStatus>>(_a[1]))); break;
        case 2: _t->d->doExecuteAction(); break;
        case 3: _t->d->doAuthorizeAction(); break;
        case 4: _t->d->actionPerformedSlot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KAuth::ActionReply>>(_a[2]))); break;
        case 5: _t->d->progressStepSlot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->d->statusChangedSlot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KAuth::Action::AuthStatus>>(_a[2]))); break;
        case 7: { bool _r = _t->kill((*reinterpret_cast<std::add_pointer_t<KillVerbosity>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->kill();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KAuth::ActionReply >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ExecuteJob::*)(const QVariantMap & )>(_a, &ExecuteJob::newData, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExecuteJob::*)(KAuth::Action::AuthStatus )>(_a, &ExecuteJob::statusChanged, 1))
            return;
    }
}

const QMetaObject *KAuth::ExecuteJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KAuth::ExecuteJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KAuth10ExecuteJobE_t>.strings))
        return static_cast<void*>(this);
    return KJob::qt_metacast(_clname);
}

int KAuth::ExecuteJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KAuth::ExecuteJob::newData(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KAuth::ExecuteJob::statusChanged(KAuth::Action::AuthStatus _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
