/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kf6auth.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "kf6authadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class Kf6authAdaptor
 */

Kf6authAdaptor::Kf6authAdaptor(KAuth::DBusHelperProxy *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

Kf6authAdaptor::~Kf6authAdaptor()
{
    // destructor
}

QByteArray Kf6authAdaptor::performAction(const QString &action, const QByteArray &callerID, const QVariantMap &details, const QByteArray &arguments, const QMap<QString,QDBusUnixFileDescriptor> &fdArguments)
{
    // handle method call org.kde.kf6auth.performAction
    return parent()->performAction(action, callerID, details, arguments, fdArguments);
}

void Kf6authAdaptor::stopAction(const QString &action)
{
    // handle method call org.kde.kf6auth.stopAction
    parent()->stopAction(action);
}


#include "moc_kf6authadaptor.cpp"
