/****************************************************************************
** Meta object code from reading C++ file 'DBusHelperProxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kauth-6.26.0/src/backends/dbus/DBusHelperProxy.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DBusHelperProxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5KAuth15DBusHelperProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto KAuth::DBusHelperProxy::qt_create_metaobjectdata<qt_meta_tag_ZN5KAuth15DBusHelperProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KAuth::DBusHelperProxy",
        "remoteSignal",
        "",
        "type",
        "action",
        "blob",
        "stopAction",
        "performAction",
        "callerID",
        "QVariantMap",
        "details",
        "arguments",
        "QMap<QString,QDBusUnixFileDescriptor>",
        "fdArguments",
        "remoteSignalReceived"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'remoteSignal'
        QtMocHelpers::SignalData<void(int, const QString &, const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 }, { QMetaType::QByteArray, 5 },
        }}),
        // Slot 'stopAction'
        QtMocHelpers::SlotData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'performAction'
        QtMocHelpers::SlotData<QByteArray(const QString &, const QByteArray &, const QVariantMap &, QByteArray, const QMap<QString,QDBusUnixFileDescriptor> &)>(7, 2, QMC::AccessPublic, QMetaType::QByteArray, {{
            { QMetaType::QString, 4 }, { QMetaType::QByteArray, 8 }, { 0x80000000 | 9, 10 }, { QMetaType::QByteArray, 11 },
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'remoteSignalReceived'
        QtMocHelpers::SlotData<void(int, const QString &, QByteArray)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 }, { QMetaType::QByteArray, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DBusHelperProxy, qt_meta_tag_ZN5KAuth15DBusHelperProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KAuth::DBusHelperProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<HelperProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KAuth15DBusHelperProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KAuth15DBusHelperProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5KAuth15DBusHelperProxyE_t>.metaTypes,
    nullptr
} };

void KAuth::DBusHelperProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DBusHelperProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->remoteSignal((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3]))); break;
        case 1: _t->stopAction((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: { QByteArray _r = _t->performAction((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QDBusUnixFileDescriptor>>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->remoteSignalReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DBusHelperProxy::*)(int , const QString & , const QByteArray & )>(_a, &DBusHelperProxy::remoteSignal, 0))
            return;
    }
}

const QMetaObject *KAuth::DBusHelperProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KAuth::DBusHelperProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KAuth15DBusHelperProxyE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    if (!strcmp(_clname, "org.kde.kf6auth.HelperProxy/0.1"))
        return static_cast< KAuth::HelperProxy*>(this);
    return HelperProxy::qt_metacast(_clname);
}

int KAuth::DBusHelperProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HelperProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void KAuth::DBusHelperProxy::remoteSignal(int _t1, const QString & _t2, const QByteArray & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
using namespace KAuth;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_DBusHelperProxy[] = {
    0xbf, 
    // "IID"
    0x02,  0x77,  'o',  'r',  'g',  '.',  'k',  'd', 
    'e',  '.',  'D',  'B',  'u',  's',  'H',  'e', 
    'l',  'p',  'e',  'r',  'P',  'r',  'o',  'x', 
    'y', 
    // "className"
    0x03,  0x6f,  'D',  'B',  'u',  's',  'H',  'e', 
    'l',  'p',  'e',  'r',  'P',  'r',  'o',  'x', 
    'y', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(KAuth::DBusHelperProxy, DBusHelperProxy, qt_pluginMetaDataV2_DBusHelperProxy)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_DBusHelperProxy[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x77,  'o',  'r',  'g',  '.',  'k',  'd', 
    'e',  '.',  'D',  'B',  'u',  's',  'H',  'e', 
    'l',  'p',  'e',  'r',  'P',  'r',  'o',  'x', 
    'y', 
    // "className"
    0x03,  0x6f,  'D',  'B',  'u',  's',  'H',  'e', 
    'l',  'p',  'e',  'r',  'P',  'r',  'o',  'x', 
    'y', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(KAuth::DBusHelperProxy, DBusHelperProxy)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
