/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kf6auth.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef KF6AUTHADAPTOR_H
#define KF6AUTHADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "backends/dbus/DBusHelperProxy.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.kf6auth
 */
class Kf6authAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.kf6auth")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.kf6auth\">\n"
"    <method name=\"performAction\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"action\"/>\n"
"      <arg direction=\"in\" type=\"ay\" name=\"callerID\"/>\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"details\"/>\n"
"      <arg direction=\"in\" type=\"ay\" name=\"arguments\"/>\n"
"      <arg direction=\"in\" type=\"a{sh}\" name=\"fdArguments\"/>\n"
"      <arg direction=\"out\" type=\"ay\" name=\"r\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In2\"/>\n"
"      <annotation value=\"QMap&lt;QString,QDBusUnixFileDescriptor&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.In4\"/>\n"
"    </method>\n"
"    <method name=\"stopAction\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"action\"/>\n"
"      <annotation value=\"true\" name=\"org.freedesktop.DBus.Method.NoReply\"/>\n"
"    </method>\n"
"    <signal name=\"remoteSignal\">\n"
"      <arg type=\"i\" name=\"type\"/>\n"
"      <arg type=\"s\" name=\"action\"/>\n"
"      <arg type=\"ay\" name=\"blob\"/>\n"
"    </signal>\n"
"  </interface>\n"
        "")
public:
    Kf6authAdaptor(KAuth::DBusHelperProxy *parent);
    ~Kf6authAdaptor() override;

    inline KAuth::DBusHelperProxy *parent() const
    { return static_cast<KAuth::DBusHelperProxy *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QByteArray performAction(const QString &action, const QByteArray &callerID, const QVariantMap &details, const QByteArray &arguments, const QMap<QString,QDBusUnixFileDescriptor> &fdArguments);
    Q_NOREPLY void stopAction(const QString &action);
Q_SIGNALS: // SIGNALS
    void remoteSignal(int type, const QString &action, const QByteArray &blob);
};

#endif
