/****************************************************************************
** Meta object code from reading C++ file 'kf6authadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "kf6authadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kf6authadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14Kf6authAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto Kf6authAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN14Kf6authAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kf6authAdaptor",
        "D-Bus Interface",
        "org.kde.kf6auth",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.kf6auth\">\n    <method name=\"perform"
        "Action\">\n      <arg direction=\"in\" type=\"s\" name=\"action\"/"
        ">\n      <arg direction=\"in\" type=\"ay\" name=\"callerID\"/>\n  "
        "    <arg direction=\"in\" type=\"a{sv}\" name=\"details\"/>\n     "
        " <arg direction=\"in\" type=\"ay\" name=\"arguments\"/>\n      <ar"
        "g direction=\"in\" type=\"a{sh}\" name=\"fdArguments\"/>\n      <a"
        "rg direction=\"out\" type=\"ay\" name=\"r\"/>\n      <annotation v"
        "alue=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In2\"/"
        ">\n      <annotation value=\"QMap&lt;QString,QDBusUnixFileDescript"
        "or&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.In4\"/>\n    </met"
        "hod>\n    <method name=\"stopAction\">\n      <arg direction=\"in\""
        " type=\"s\" name=\"action\"/>\n      <annotation value=\"true\" na"
        "me=\"org.freedesktop.DBus.Method.NoReply\"/>\n    </method>\n    <"
        "signal name=\"remoteSignal\">\n      <arg type=\"i\" name=\"type\""
        "/>\n      <arg type=\"s\" name=\"action\"/>\n      <arg type=\"ay\""
        " name=\"blob\"/>\n    </signal>\n  </interface>\n",
        "remoteSignal",
        "",
        "type",
        "action",
        "blob",
        "performAction",
        "callerID",
        "QVariantMap",
        "details",
        "arguments",
        "QMap<QString,QDBusUnixFileDescriptor>",
        "fdArguments",
        "stopAction",
        "Q_NOREPLY"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'remoteSignal'
        QtMocHelpers::SignalData<void(int, const QString &, const QByteArray &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::QString, 8 }, { QMetaType::QByteArray, 9 },
        }}),
        // Slot 'performAction'
        QtMocHelpers::SlotData<QByteArray(const QString &, const QByteArray &, const QVariantMap &, const QByteArray &, const QMap<QString,QDBusUnixFileDescriptor> &)>(10, 6, QMC::AccessPublic, QMetaType::QByteArray, {{
            { QMetaType::QString, 8 }, { QMetaType::QByteArray, 11 }, { 0x80000000 | 12, 13 }, { QMetaType::QByteArray, 14 },
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'stopAction'
        QtMocHelpers::SlotData<void(const QString &)>(17, 18, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<Kf6authAdaptor, qt_meta_tag_ZN14Kf6authAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Kf6authAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14Kf6authAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14Kf6authAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14Kf6authAdaptorE_t>.metaTypes,
    nullptr
} };

void Kf6authAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Kf6authAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->remoteSignal((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3]))); break;
        case 1: { QByteArray _r = _t->performAction((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QDBusUnixFileDescriptor>>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->stopAction((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Kf6authAdaptor::*)(int , const QString & , const QByteArray & )>(_a, &Kf6authAdaptor::remoteSignal, 0))
            return;
    }
}

const QMetaObject *Kf6authAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kf6authAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14Kf6authAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int Kf6authAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void Kf6authAdaptor::remoteSignal(int _t1, const QString & _t2, const QByteArray & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
