/****************************************************************************
** Meta object code from reading C++ file 'action.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kauth-6.26.0/src/action.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'action.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5KAuth6ActionE_t {};
} // unnamed namespace

template <> constexpr inline auto KAuth::Action::qt_create_metaobjectdata<qt_meta_tag_ZN5KAuth6ActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KAuth::Action",
        "AuthStatus",
        "DeniedStatus",
        "ErrorStatus",
        "InvalidStatus",
        "AuthorizedStatus",
        "AuthRequiredStatus",
        "UserCancelledStatus",
        "ExecutionMode",
        "ExecuteMode",
        "AuthorizeOnlyMode",
        "AuthDetail",
        "DetailOther",
        "DetailMessage"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AuthStatus'
        QtMocHelpers::EnumData<enum AuthStatus>(1, 1, QMC::EnumFlags{}).add({
            {    2, AuthStatus::DeniedStatus },
            {    3, AuthStatus::ErrorStatus },
            {    4, AuthStatus::InvalidStatus },
            {    5, AuthStatus::AuthorizedStatus },
            {    6, AuthStatus::AuthRequiredStatus },
            {    7, AuthStatus::UserCancelledStatus },
        }),
        // enum 'ExecutionMode'
        QtMocHelpers::EnumData<enum ExecutionMode>(8, 8, QMC::EnumFlags{}).add({
            {    9, ExecutionMode::ExecuteMode },
            {   10, ExecutionMode::AuthorizeOnlyMode },
        }),
        // enum 'AuthDetail'
        QtMocHelpers::EnumData<enum AuthDetail>(11, 11, QMC::EnumIsScoped).add({
            {   12, AuthDetail::DetailOther },
            {   13, AuthDetail::DetailMessage },
        }),
    };
    return QtMocHelpers::metaObjectData<Action, qt_meta_tag_ZN5KAuth6ActionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KAuth::Action::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KAuth6ActionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KAuth6ActionE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5KAuth6ActionE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
